# Generated by Django 4.0.1 on 2022-01-22 12:31

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('django_pg_trunk', '0001_create_pg_stat_statements_extension'),
    ]

    operations = [
        migrations.CreateModel(
            name='QueryStatistic',
            fields=[
                ('dbid', models.BigIntegerField(help_text='OID of database in which the statement was executed')),
                ('userid', models.BigIntegerField(help_text='OID of user who executed the statement')),
                ('queryid', models.BigIntegerField(help_text="Internal hash code, computed from the statement's parse tree", primary_key=True, serialize=False)),
                ('query', models.TextField(help_text='Text of a representative statement')),
                ('rows', models.BigIntegerField(help_text='Total number of rows retrieved or affected by the statement')),
                ('calls', models.BigIntegerField(help_text='Number of times executed')),
                ('total_exec_time', models.FloatField(help_text='Total time spent executing the statement, in milliseconds')),
                ('min_exec_time', models.FloatField(help_text='Minimum time spent executing the statement, in milliseconds')),
                ('max_exec_time', models.FloatField(help_text='Maximum time spent executing the statement, in milliseconds')),
                ('mean_exec_time', models.FloatField(help_text='Mean time spent executing the statement, in milliseconds')),
                ('stddev_exec_time', models.FloatField(help_text='Population standard deviation of time spent executing the statement, in milliseconds')),
                ('plans', models.BigIntegerField(help_text='Number of times the statement was planned (if pg_stat_statements.track_planning is enabled, otherwise zero)')),
                ('total_plan_time', models.BigIntegerField(help_text='Total time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)')),
                ('min_plan_time', models.BigIntegerField(help_text='Minimum time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)')),
                ('max_plan_time', models.BigIntegerField(help_text='Maximum time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)')),
                ('mean_plan_time', models.BigIntegerField(help_text='Mean time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)')),
                ('stddev_plan_time', models.BigIntegerField(help_text='Population standard deviation of time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)')),
                ('shared_blks_hit', models.BigIntegerField(help_text='Total number of shared block cache hits by the statement')),
                ('shared_blks_read', models.BigIntegerField(help_text='Total number of shared blocks read by the statement')),
                ('shared_blks_dirtied', models.BigIntegerField(help_text='Total number of shared blocks dirtied by the statement')),
                ('shared_blks_written', models.BigIntegerField(help_text='Total number of shared blocks written by the statement')),
                ('local_blks_hit', models.BigIntegerField(help_text='Total number of local block cache hits by the statement')),
                ('local_blks_read', models.BigIntegerField(help_text='Total number of local blocks read by the statement')),
                ('local_blks_dirtied', models.BigIntegerField(help_text='Total number of local blocks dirtied by the statement')),
                ('local_blks_written', models.BigIntegerField(help_text='Total number of local blocks written by the statement')),
                ('temp_blks_read', models.BigIntegerField(help_text='Total number of temp blocks read by the statement')),
                ('temp_blks_written', models.BigIntegerField(help_text='Total number of temp blocks written by the statement')),
                ('blk_read_time', models.FloatField(help_text='Total time the statement spent reading blocks, in milliseconds (if track_io_timing is enabled, otherwise zero)')),
                ('blk_write_time', models.FloatField(help_text='Total time the statement spent writing blocks, in milliseconds (if track_io_timing is enabled, otherwise zero)')),
                ('wal_records', models.BigIntegerField(help_text='Total number of WAL records generated by the statement')),
                ('wal_fpi', models.BigIntegerField(help_text='Total number of WAL full page images generated by the statement')),
                ('wal_bytes', models.DecimalField(decimal_places=2, help_text='Total amount of WAL generated by the statement in bytes', max_digits=20)),
            ],
            options={
                'verbose_name': 'Query Statistic',
                'verbose_name_plural': 'Query Statistics',
                'db_table': 'pg_stat_statements',
                'ordering': ('-mean_exec_time',),
                'managed': False,
            },
        ),
    ]



# Generated by Django 4.0.1 on 2022-01-21 16:45

from django.contrib.postgres.operations import CreateExtension
from django.db import migrations


class PgStatStatementsExtension(CreateExtension):

    def __init__(self):
        self.name = 'pg_stat_statements'


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        PgStatStatementsExtension()
    ]



from django.contrib import admin
from django.contrib.admin.options import IncorrectLookupParameters
from django.contrib.admin.views.main import ChangeList
from django.core.exceptions import ValidationError

from django_pg_trunk.models import QueryStatistic
from django_pg_trunk.utils import get_current_database_id


class DbidListFilter(admin.SimpleListFilter):
    parameter_name = "dbid"

    def __init__(self, request, params, model, model_admin):
        self.title = "Database OID"
        super().__init__(request, params, model, model_admin)

    def expected_parameters(self):
        return ["dbid"]

    def lookups(self, request, model_admin):
        values = model_admin.model.objects.distinct("dbid").order_by("dbid").values_list("dbid", flat=True)
        lookups = []
        for value in values:
            verbose_name = str(value)
            if value == get_current_database_id():
                verbose_name += " (Current Database)"

            lookups.append((value, verbose_name))
        return lookups

    def queryset(self, request, queryset):
        try:
            return queryset.filter(**self.used_parameters)
        except (ValueError, ValidationError) as e:
            # Fields may raise a ValueError or ValidationError when converting
            # the parameters to the correct type.
            raise IncorrectLookupParameters(e)


class QueryStatisticAdmin(admin.ModelAdmin):
    fieldsets = (
        ("Statement", {
            "fields": ("userid", "dbid", "query", "rows", "calls",)
        }),
        ("Exec Time", {
            "fields": ("total_exec_time", "min_exec_time", "max_exec_time", "mean_exec_time", "stddev_exec_time",)
        }),
        ("Plans", {
            "fields": ("plans", "total_plan_time", "min_plan_time", "max_plan_time", "mean_plan_time", "stddev_plan_time",)
        }),
        ("Shared Blocks", {
            "fields": ("shared_blks_hit", "shared_blks_read", "shared_blks_dirtied", "shared_blks_written",)
        }),
        ("Local Blocks", {
            "fields": ("local_blks_hit", "local_blks_read", "local_blks_dirtied", "local_blks_written",)
        }),
        ("Temp Blocks", {
            "fields": ("temp_blks_read", "temp_blks_written",)
        }),
        ("Block Read/Write", {
            "fields": ("blk_read_time", "blk_write_time",)
        }),
        ("WAL", {
            "fields": ("wal_records", "wal_fpi", "wal_bytes",)
        }),
    )
    list_display = ("queryid", "dbid", "userid", "query", "mean_exec_time", "rows", "calls",)
    search_fields = ("query",)
    list_filter = (DbidListFilter,)

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False

    def get_changelist(self, request, **kwargs):
        class QueryStatisticChangeList(ChangeList):
            def url_for_result(self, result):
                url = super().url_for_result(result)
                return url.replace(str(result.queryid), f"{result.queryid}_{result.dbid}_{result.userid}")

        return QueryStatisticChangeList

    def get_object(self, request, object_id, from_field=None):
        queryset = self.get_queryset(request)
        queryid, dbid, userid = object_id.split("_")
        try:
            return queryset.get(queryid=queryid, dbid=dbid, userid=userid)
        except (queryset.model.DoesNotExist, ValidationError, ValueError):
            return None


admin.site.register(QueryStatistic, QueryStatisticAdmin)



from django.db import models


class QueryStatistic(models.Model):
    dbid = models.BigIntegerField(help_text="OID of database in which the statement was executed")
    userid = models.BigIntegerField(help_text="OID of user who executed the statement")
    queryid = models.BigIntegerField(primary_key=True, help_text="Internal hash code, computed from the statement's parse tree")
    query = models.TextField(help_text="Text of a representative statement")
    rows = models.BigIntegerField(help_text="Total number of rows retrieved or affected by the statement")
    calls = models.BigIntegerField(help_text="Number of times executed")

    total_exec_time = models.FloatField(help_text="Total time spent executing the statement, in milliseconds")
    min_exec_time = models.FloatField(help_text="Minimum time spent executing the statement, in milliseconds")
    max_exec_time = models.FloatField(help_text="Maximum time spent executing the statement, in milliseconds")
    mean_exec_time = models.FloatField(help_text="Mean time spent executing the statement, in milliseconds")
    stddev_exec_time = models.FloatField(help_text="Population standard deviation of time spent executing the statement, in milliseconds")

    plans = models.BigIntegerField(help_text="Number of times the statement was planned (if pg_stat_statements.track_planning is enabled, otherwise zero)")
    total_plan_time = models.BigIntegerField(help_text="Total time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)")
    min_plan_time = models.BigIntegerField(help_text="Minimum time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)")
    max_plan_time = models.BigIntegerField(help_text="Maximum time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)")
    mean_plan_time = models.BigIntegerField(help_text="Mean time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)")
    stddev_plan_time = models.BigIntegerField(help_text="Population standard deviation of time spent planning the statement, in milliseconds (if pg_stat_statements.track_planning is enabled, otherwise zero)")

    shared_blks_hit = models.BigIntegerField(help_text="Total number of shared block cache hits by the statement")
    shared_blks_read = models.BigIntegerField(help_text="Total number of shared blocks read by the statement")
    shared_blks_dirtied = models.BigIntegerField(help_text="Total number of shared blocks dirtied by the statement")
    shared_blks_written = models.BigIntegerField(help_text="Total number of shared blocks written by the statement")

    local_blks_hit = models.BigIntegerField(help_text="Total number of local block cache hits by the statement")
    local_blks_read = models.BigIntegerField(help_text="Total number of local blocks read by the statement")
    local_blks_dirtied = models.BigIntegerField(help_text="Total number of local blocks dirtied by the statement")
    local_blks_written = models.BigIntegerField(help_text="Total number of local blocks written by the statement")

    temp_blks_read = models.BigIntegerField(help_text="Total number of temp blocks read by the statement")
    temp_blks_written = models.BigIntegerField(help_text="Total number of temp blocks written by the statement")

    blk_read_time = models.FloatField(help_text="Total time the statement spent reading blocks, in milliseconds (if track_io_timing is enabled, otherwise zero)")
    blk_write_time = models.FloatField(help_text="Total time the statement spent writing blocks, in milliseconds (if track_io_timing is enabled, otherwise zero)")

    wal_records = models.BigIntegerField(help_text="Total number of WAL records generated by the statement")
    wal_fpi = models.BigIntegerField(help_text="Total number of WAL full page images generated by the statement")
    wal_bytes = models.DecimalField(max_digits=20, decimal_places=2, help_text="Total amount of WAL generated by the statement in bytes")

    class Meta:
        managed = False
        db_table = "pg_stat_statements"
        ordering = ("-mean_exec_time",)
        verbose_name = "Query Statistic"
        verbose_name_plural = "Query Statistics"



from django.test import TestCase

from django_pg_trunk.models import QueryStatistic
from django_pg_trunk.utils import get_current_database_id


class PgTrunkTests(TestCase):
    def test_query_statistic_model(self):
        # Test fetching QueryStatistic objects from pg_stat_statements database.
        self.assertIsNotNone(QueryStatistic.objects.first())

        # Count will be increased since statistics of the query that
        # calculates the count will be added to the pg_stat_statements.
        query_statistic_count = QueryStatistic.objects.count()
        self.assertEqual(QueryStatistic.objects.count(), query_statistic_count + 1)

    def test_get_current_database_id(self):
        database_id = get_current_database_id()
        self.assertTrue(
            QueryStatistic.objects.filter(query="SELECT current_database()", dbid=database_id).exists()
        )



from django.db import connection


def get_current_database_id():
    with connection.cursor() as cursor:
        cursor.execute("SELECT current_database()")
        database_name = cursor.fetchone()[0]

        cursor.execute("SELECT oid FROM pg_database WHERE datname = %s", [database_name])
        database_id = cursor.fetchone()[0]

        return database_id



import os
from setuptools import find_packages, setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

setup(
    name="django-pg-trunk",
    version="0.1.2",
    packages=find_packages(exclude=["test*"]),
    include_package_data=True,
    install_requires=[
        "Django>=2.2",
        "psycopg2-binary<2.9"
    ],
    license="Apache-2.0",
    description="A PostgreSQL profiler for Django that uses pg_stat_statements extension",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Hipo",
    author_email="faruk@hipo.biz",
    url="https://github.com/Hipo/django-pg-trunk",
    python_requires=">=3.7",
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)
